@@ -1,4 +1,4 @@
 package T3;
-use Object::Simple( a => 'B1' );
+use Object::Simple( a => ['B1'] );
 
 Object::Simple->end;
